/*
 CardInspectorController.h - card inspector controller interface for Popup.app
 Copyright (C) 2004 Rob Burns
 February,28 2004
 
 This application is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or any later version.

 This application is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

 You should have received a copy of the GNU General Public
 License along with this application; if not, write to the Free
 Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA.
*/


#ifndef _CARD_INSPECTOR_CONTROLLER_H_
#define _CARD_INSPECTOR_CONTROLLER_H_

#ifdef __APPLE__
  #import <Cocoa/Cocoa.h>
  #include "GNUstep.h"
#else
  #include <Foundation/Foundation.h>
  #include <AppKit/AppKit.h>
#endif

@class CardModel;
@class StackModel;

@interface CardInspectorController : NSWindowController
{
  id tabButton;
  id tabView;
  
  id frontField;
  id backField;
  
  id posLabel;
  id posField;
  id timeLabel;
  id timeField;
  
  id grammarPopup;
  id usagePopup;
  id noteField;
  
  id historyView;
  id passLabel;
  id passField;  
  id failLabel;
  id failField;
  id resultGraph;
  
  CardModel *card;
  StackModel *stack;
  
  NSImage *passImage;
  NSImage *failImage;
}

+ (id) sharedInspector;

- (void) dealloc;
- (id) init;
- (void) awakeFromNib;


// **************
// Action methods
// **************


- (void) cardChanged: (NSNotification *)aNotification;
- (void) stackChanged: (NSNotification *)aNotification;
- (void) changeTabView: (id)sender;


// ***************
// Private methods
// ***************


- (void) _updateResults;


// ****************************
// NSTableView delegate methods
// ****************************


               
               
// **********************************
// NSTableDataSource Protocol methods
// **********************************


- (int) numberOfRowsInTableView: (NSTableView *)aTableView;

- (id)            tableView: (NSTableView *)aTableView
  objectValueForTableColumn: (NSTableColumn *)aTableColumn 
                        row: (int)rowIndex;


@end

#endif // _CARD_INSPECTOR_CONTROLLER_H_

